package workspace

import (
	"encoding/json"

	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
	"github.com/google/uuid"
)

// Workspace is a struct that represents a workspace
type Workspace struct {
	utils.AbstractObject         // AbstractObject contains the basic fields of an object (id, name)
	resources.ResourceSet        // ResourceSet contains the resources of the workspace (data, datacenter, processing, storage, workflow)
	IsContextual          bool   `json:"is_contextual" bson:"is_contextual" default:"false"` // IsContextual is a flag that indicates if the workspace is contextual
	Active                bool   `json:"active" bson:"active" default:"false"`               // Active is a flag that indicates if the workspace is active
	Shared                string `json:"shared,omitempty" bson:"shared,omitempty"`           // Shared is the ID of the shared workspace
}

func (ao *Workspace) GetID() string {
	return ao.UUID
}

func (r *Workspace) GenerateID() {
	if r.UUID == "" {
		r.UUID = uuid.New().String()
	}
}

func (d *Workspace) GetName() string {
	return d.Name
}

func (d *Workspace) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := New()                      // Create a new instance of the accessor
	data.Init(tools.WORKSPACE, caller) // Initialize the accessor with the WORKSPACE model type
	return data
}

// New creates a new instance of the workspaceMongoAccessor from a map
func (dma *Workspace) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

// Serialize returns the workspaceMongoAccessor as a map
func (dma *Workspace) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
